@@ -66,16 +66,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <glib/gi18n.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkiconfactory.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreestore.h>
-#include <gtk/gtkcellrenderertoggle.h>
-#include <gtk/gtkaccelmap.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkcellrendereraccel.h>
+#include <gtk/gtk.h>
 
 #include "resources.h"
 #include "anjuta-ui.h"
@@ -93,6 +84,7 @@ enum {
 	COLUMN_PIXBUF,
 	COLUMN_ACTION_LABEL,
 	COLUMN_VISIBLE,
+	COLUMN_SHOW_VISIBLE,
 	COLUMN_SENSITIVE,
 	COLUMN_ACTION,
 	COLUMN_GROUP,
@@ -363,6 +355,7 @@ anjuta_ui_init (AnjutaUI *ui)
 								G_TYPE_STRING,
 								G_TYPE_BOOLEAN,
 								G_TYPE_BOOLEAN,
+								G_TYPE_BOOLEAN,
 								G_TYPE_OBJECT,
 								G_TYPE_STRING);
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE(store), COLUMN_ACTION_LABEL,
@@ -533,6 +526,7 @@ anjuta_ui_add_action_group (AnjutaUI *ui,
 						COLUMN_PIXBUF, pixbuf,
 						COLUMN_ACTION_LABEL, action_group_label,
 						COLUMN_GROUP, action_group_name,
+						COLUMN_SHOW_VISIBLE, FALSE,
 						-1);
 	for (l = actions; l; l = l->next)
 	{
@@ -570,6 +564,7 @@ anjuta_ui_add_action_group (AnjutaUI *ui,
 									COLUMN_PIXBUF, pixbuf,
 									COLUMN_ACTION_LABEL, action_label,
 									COLUMN_VISIBLE, gtk_action_get_visible (action),
+									COLUMN_SHOW_VISIBLE, TRUE,
 									COLUMN_SENSITIVE, gtk_action_get_sensitive(action),
 									COLUMN_ACTION, action,
 									COLUMN_GROUP, action_group_name,
@@ -584,6 +579,7 @@ anjuta_ui_add_action_group (AnjutaUI *ui,
 			gtk_tree_store_set (GTK_TREE_STORE (ui->priv->model), &iter,
 								COLUMN_ACTION_LABEL, action_label,
 								COLUMN_VISIBLE, gtk_action_get_visible (action),
+								COLUMN_SHOW_VISIBLE, TRUE,
 								COLUMN_SENSITIVE, gtk_action_get_sensitive (action),
 								COLUMN_ACTION, action,
 								COLUMN_GROUP, action_group_name,
@@ -868,6 +864,8 @@ anjuta_ui_get_accel_editor (AnjutaUI *ui)
 													   renderer,
 													   "active",
 													   COLUMN_VISIBLE,
+													   "visible",
+													   COLUMN_SHOW_VISIBLE,
 													   NULL);
 	/* gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE); */
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
@@ -956,28 +954,54 @@ anjuta_ui_get_accel_file (void)
 }
 
 void
-anjuta_ui_load_accels (void)
+anjuta_ui_load_accels (const gchar *filename)
 {
-	gchar *filename;
-
-	filename = anjuta_ui_get_accel_file ();
-	if (filename != NULL)
+	if (filename)
 	{
 		gtk_accel_map_load (filename);
-		g_free (filename);
+	}
+	else
+	{
+		gchar *def_filename = anjuta_ui_get_accel_file ();
+		if (def_filename != NULL)
+		{
+			gtk_accel_map_load (def_filename);
+			g_free (def_filename);
+		}
 	}
 }
 
 void
-anjuta_ui_save_accels (void)
+anjuta_ui_save_accels (const gchar *filename)
 {
-	gchar *filename;
-
-	filename = anjuta_ui_get_accel_file ();
-	if (filename != NULL)
+	if (filename)
 	{
 		gtk_accel_map_save (filename);
-		g_free (filename);
+	}
+	else
+	{
+        	gchar * def_filename = anjuta_ui_get_accel_file ();
+
+		if (def_filename != NULL)
+		{
+			gtk_accel_map_save (def_filename);
+			g_free (def_filename);
+		}
 	}
 }
 
+static void anjuta_ui_remove_accel (AnjutaUI *ui,
+     const gchar *accel_path, guint accel_key,
+     GdkModifierType accel_mods, gboolean changed)
+{
+    gtk_accel_group_disconnect_key (anjuta_ui_get_accel_group(ui), accel_key, accel_mods);
+}
+
+void
+anjuta_ui_unload_accels (AnjutaUI *ui)
+{
+    anjuta_ui_save_accels (NULL);
+    gtk_accel_map_foreach_unfiltered (ui, (GtkAccelMapForeach) anjuta_ui_remove_accel);
+} 
+
+
