@@ -1,7 +1,6 @@
 
 /* Interfaces global include file */
 
-#include "ianjuta-bookmark.h"
 #include "ianjuta-buildable.h"
 #include "ianjuta-builder.h"
 #include "ianjuta-debug-manager.h"
@@ -27,8 +26,10 @@
 #include "ianjuta-editor-line-mode.h"
 #include "ianjuta-editor-search.h"
 #include "ianjuta-editor-selection.h"
+#include "ianjuta-editor-tip.h"
 #include "ianjuta-editor-view.h"
 #include "ianjuta-editor-zoom.h"
+#include "ianjuta-environment.h"
 #include "ianjuta-file.h"
 #include "ianjuta-file-loader.h"
 #include "ianjuta-file-manager.h"
@@ -47,7 +48,9 @@
 #include "ianjuta-plugin-factory.h"
 #include "ianjuta-preferences.h"
 #include "ianjuta-print.h"
+#include "ianjuta-project-backend.h"
 #include "ianjuta-project-manager.h"
+#include "ianjuta-provider.h"
 #include "ianjuta-stream.h"
 #include "ianjuta-stream-loader.h"
 #include "ianjuta-stream-savable.h"
