@@ -38,7 +38,7 @@ static void sourceview_cell_instance_init(SourceviewCell *sp);
 static void sourceview_cell_finalize(GObject *object);
 
 struct _SourceviewCellPrivate {
-	GtkTextIter* iter;
+	GtkTextMark* mark;
 	GtkTextView* view;
 	GtkTextBuffer* buffer;
 };
@@ -69,8 +69,6 @@ sourceview_cell_finalize(GObject *object)
 	SourceviewCell *cobj;
 	cobj = SOURCEVIEW_CELL(object);
 	
-	gtk_text_iter_free(cobj->priv->iter);
-	
 	g_slice_free(SourceviewCellPrivate, cobj->priv);
 	G_OBJECT_CLASS(sourceview_cell_parent_class)->finalize(object);
 }
@@ -83,23 +81,25 @@ sourceview_cell_new(GtkTextIter* iter, GtkTextView* view)
 	obj = SOURCEVIEW_CELL(g_object_new(SOURCEVIEW_TYPE_CELL, NULL));
 	
 	obj->priv->buffer = gtk_text_view_get_buffer(view);
-	obj->priv->iter = gtk_text_iter_copy (iter);
+	obj->priv->mark = gtk_text_buffer_create_mark (obj->priv->buffer, NULL, iter, FALSE);
 	obj->priv->view = view;
 	
 	return obj;
 }
 
-GtkTextIter*
-sourceview_cell_get_iter (SourceviewCell* cell)
+void
+sourceview_cell_get_iter (SourceviewCell* cell, GtkTextIter* iter)
 {
-	return cell->priv->iter;
+	gtk_text_buffer_get_iter_at_mark (cell->priv->buffer, iter, cell->priv->mark);
 }
 
 static gchar*
 icell_get_character(IAnjutaEditorCell* icell, GError** e)
 {
 	SourceviewCell* cell = SOURCEVIEW_CELL(icell);
-	gunichar c = gtk_text_iter_get_char (cell->priv->iter);
+	GtkTextIter iter;
+	gtk_text_buffer_get_iter_at_mark (cell->priv->buffer, &iter, cell->priv->mark);	
+	gunichar c = gtk_text_iter_get_char (&iter);
 	gchar* outbuf = g_new0(gchar, 6);
 	g_unichar_to_utf8 (c, outbuf);
 	return outbuf;
@@ -120,7 +120,9 @@ static gchar
 icell_get_char(IAnjutaEditorCell* icell, gint index, GError** e)
 {
 	SourceviewCell* cell = SOURCEVIEW_CELL(icell);
-	gunichar c = gtk_text_iter_get_char (cell->priv->iter);
+	GtkTextIter iter;
+	gtk_text_buffer_get_iter_at_mark (cell->priv->buffer, &iter, cell->priv->mark);
+	gunichar c = gtk_text_iter_get_char (&iter);
 	gchar* outbuf = g_new0(gchar, 6);
 	gint len = g_unichar_to_utf8 (c, outbuf);
 	gchar retval;
@@ -148,169 +150,150 @@ icell_iface_init(IAnjutaEditorCellIface* iface)
 	iface->get_attribute = icell_get_attribute;
 }
 
-static
-GtkTextAttributes* get_attributes(GtkTextIter* iter, GtkTextView* view)
-{
-	GtkTextAttributes* atts = gtk_text_view_get_default_attributes(view);
-	gtk_text_iter_get_attributes(iter, atts);
-	return atts;
-}
-
-static gchar*
-icell_style_get_font_description(IAnjutaEditorCellStyle* icell_style, GError ** e)
-{
-	const gchar* font;
-	SourceviewCell* cell = SOURCEVIEW_CELL(icell_style);
-	GtkTextAttributes* atts = get_attributes(cell->priv->iter, 
-																					 cell->priv->view);
-	font = pango_font_description_to_string(atts->font);
-	g_free(atts);
-	return g_strdup(font);
-}
-
-static gchar*
-icell_style_get_color(IAnjutaEditorCellStyle* icell_style, GError ** e)
-{
-	gchar* color;
-	SourceviewCell* cell = SOURCEVIEW_CELL(icell_style);
-	GtkTextAttributes* atts = get_attributes(cell->priv->iter, cell->priv->view);
-	color = anjuta_util_string_from_color(atts->appearance.fg_color.red,
-																				atts->appearance.fg_color.green, atts->appearance.fg_color.blue);
-	g_free(atts);
-	return color;
-}
-
-static gchar*
-icell_style_get_background_color(IAnjutaEditorCellStyle* icell_style, GError ** e)
-{
-	gchar* color;
-	SourceviewCell* cell = SOURCEVIEW_CELL(icell_style);
-	GtkTextAttributes* atts = get_attributes(cell->priv->iter, cell->priv->view);
-	color = anjuta_util_string_from_color(atts->appearance.bg_color.red,
-																				atts->appearance.bg_color.green, atts->appearance.bg_color.blue);
-	g_free(atts);
-	return color;
-}
-
-static void
-icell_style_iface_init(IAnjutaEditorCellStyleIface* iface)
-{
-	iface->get_font_description = icell_style_get_font_description;
-	iface->get_color = icell_style_get_color;
-	iface->get_background_color = icell_style_get_background_color;
-}
-
 static gboolean
-iiter_first(IAnjutaIterable* iter, GError** e)
+iiter_first(IAnjutaIterable* iiter, GError** e)
 {
-	SourceviewCell* cell = SOURCEVIEW_CELL(iter);
-	gboolean retval = gtk_text_iter_is_start (cell->priv->iter);
+	SourceviewCell* cell = SOURCEVIEW_CELL(iiter);
+	GtkTextIter iter;
+	gtk_text_buffer_get_iter_at_mark (cell->priv->buffer, &iter, cell->priv->mark);
+	gboolean retval = gtk_text_iter_is_start (&iter);
 	if (!retval)
-		gtk_text_iter_set_offset (cell->priv->iter, 0);
+	{
+		gtk_text_iter_set_offset (&iter, 0);
+		gtk_text_buffer_move_mark (cell->priv->buffer, cell->priv->mark, &iter);
+	}
 	return retval;
 }
 
 static gboolean
-iiter_next(IAnjutaIterable* iter, GError** e)
+iiter_next(IAnjutaIterable* iiter, GError** e)
 {
-	SourceviewCell* cell = SOURCEVIEW_CELL(iter);
-	
-	return gtk_text_iter_forward_char(cell->priv->iter);
-}
-
-static gboolean
-iiter_previous(IAnjutaIterable* iter, GError** e)
-{
-	SourceviewCell* cell = SOURCEVIEW_CELL(iter);
-	
-	return gtk_text_iter_backward_char(cell->priv->iter);
-}
-
-static gboolean
-iiter_last(IAnjutaIterable* iter, GError** e)
-{
-	SourceviewCell* cell = SOURCEVIEW_CELL(iter);
-	gboolean retval = gtk_text_iter_is_end (cell->priv->iter);
+	SourceviewCell* cell = SOURCEVIEW_CELL(iiter);
+	GtkTextIter iter;
+	gtk_text_buffer_get_iter_at_mark (cell->priv->buffer, &iter, cell->priv->mark);
+	gboolean retval = gtk_text_iter_forward_char(&iter);
 	if (retval)
-		gtk_text_iter_forward_to_end(cell->priv->iter);
+			gtk_text_buffer_move_mark (cell->priv->buffer, cell->priv->mark, &iter);
+	return retval;
+}
+
+static gboolean
+iiter_previous(IAnjutaIterable* iiter, GError** e)
+{
+	SourceviewCell* cell = SOURCEVIEW_CELL(iiter);
+	GtkTextIter iter;
+	gtk_text_buffer_get_iter_at_mark (cell->priv->buffer, &iter, cell->priv->mark);
+	gboolean retval = gtk_text_iter_backward_char(&iter);
+	if (retval)
+			gtk_text_buffer_move_mark (cell->priv->buffer, cell->priv->mark, &iter);
+	return retval;
+}
+
+static gboolean
+iiter_last(IAnjutaIterable* iiter, GError** e)
+{
+	SourceviewCell* cell = SOURCEVIEW_CELL(iiter);
+	GtkTextIter iter;
+	gtk_text_buffer_get_iter_at_mark (cell->priv->buffer, &iter, cell->priv->mark);
+	gboolean retval = gtk_text_iter_is_end (&iter);
+	if (!retval)
+	{
+		gtk_text_iter_forward_to_end(&iter);
+		gtk_text_buffer_move_mark (cell->priv->buffer, cell->priv->mark, &iter);
+	}
 	return retval;
 }
 
 static void
-iiter_foreach(IAnjutaIterable* iter, GFunc callback, gpointer data, GError** e)
+iiter_foreach(IAnjutaIterable* iiter, GFunc callback, gpointer data, GError** e)
 {
-	SourceviewCell* cell = SOURCEVIEW_CELL(iter);
+	SourceviewCell* cell = SOURCEVIEW_CELL(iiter);
+	GtkTextIter iter;
+	gtk_text_buffer_get_iter_at_mark (cell->priv->buffer, &iter, cell->priv->mark);
+	GtkTextMark* cached = gtk_text_buffer_create_mark (cell->priv->buffer, NULL, &iter, FALSE);
 	
-	gint saved_offset;
+	iiter_first (IANJUTA_ITERABLE(cell), NULL);
 	
-	saved_offset = gtk_text_iter_get_offset (cell->priv->iter);
-	gtk_text_iter_set_offset(cell->priv->iter, 0);
-	while (gtk_text_iter_forward_char(cell->priv->iter))
+	while (iiter_next(IANJUTA_ITERABLE(cell), NULL))
 	{
 		(*callback)(cell, data);
 	}
-	gtk_text_iter_set_offset(cell->priv->iter, saved_offset);
+	gtk_text_buffer_delete_mark (cell->priv->buffer, cell->priv->mark);
+	cell->priv->mark = cached;
 }
 
 static gboolean
-iiter_set_position (IAnjutaIterable* iter, gint position, GError** e)
+iiter_set_position (IAnjutaIterable* iiter, gint position, GError** e)
 {
-	SourceviewCell* cell = SOURCEVIEW_CELL(iter);
-	
-	gtk_text_iter_set_offset (cell->priv->iter, position);
+	SourceviewCell* cell = SOURCEVIEW_CELL(iiter);
+	GtkTextIter iter;
+	gtk_text_buffer_get_iter_at_mark (cell->priv->buffer, &iter, cell->priv->mark);
+	gtk_text_buffer_move_mark (cell->priv->buffer, cell->priv->mark, &iter);
 	return TRUE;
 }
 
 static gint
-iiter_get_position(IAnjutaIterable* iter, GError** e)
+iiter_get_position(IAnjutaIterable* iiter, GError** e)
 {
-	SourceviewCell* cell = SOURCEVIEW_CELL(iter);
-	return gtk_text_iter_get_offset(cell->priv->iter);
+	SourceviewCell* cell = SOURCEVIEW_CELL(iiter);
+	GtkTextIter iter;
+	gtk_text_buffer_get_iter_at_mark (cell->priv->buffer, &iter, cell->priv->mark);
+	return gtk_text_iter_get_offset(&iter);
 }
 
 static gint
-iiter_get_length(IAnjutaIterable* iter, GError** e)
+iiter_get_length(IAnjutaIterable* iiter, GError** e)
 {
-	SourceviewCell* cell = SOURCEVIEW_CELL(iter);
+	SourceviewCell* cell = SOURCEVIEW_CELL(iiter);
 	
-	return gtk_text_buffer_get_char_count (gtk_text_iter_get_buffer(cell->priv->iter));
+	return gtk_text_buffer_get_char_count (cell->priv->buffer);
 }
 
 static IAnjutaIterable *
-iiter_clone (IAnjutaIterable *iter, GError **e)
+iiter_clone (IAnjutaIterable *iiter, GError **e)
 {
-	SourceviewCell* cell = SOURCEVIEW_CELL(iter);
-	
-	return IANJUTA_ITERABLE (sourceview_cell_new (cell->priv->iter, cell->priv->view));
+	SourceviewCell* cell = SOURCEVIEW_CELL(iiter);
+	GtkTextIter iter;
+	gtk_text_buffer_get_iter_at_mark (cell->priv->buffer, &iter, cell->priv->mark);
+	return IANJUTA_ITERABLE (sourceview_cell_new (&iter, cell->priv->view));
 }
 
 static void
-iiter_assign (IAnjutaIterable *iter, IAnjutaIterable *src_iter, GError **e)
+iiter_assign (IAnjutaIterable *iiter, IAnjutaIterable *src_iter, GError **e)
 {
-	SourceviewCell* cell = SOURCEVIEW_CELL(iter);
+	SourceviewCell* cell = SOURCEVIEW_CELL(iiter);
 	SourceviewCell* src_cell = SOURCEVIEW_CELL(src_iter);
-	
-	gtk_text_iter_free(cell->priv->iter);
-	cell->priv->iter = gtk_text_iter_copy (src_cell->priv->iter);
+	GtkTextIter iter;
+	gtk_text_buffer_get_iter_at_mark (src_cell->priv->buffer, &iter, src_cell->priv->mark);
+	gtk_text_buffer_move_mark (cell->priv->buffer, cell->priv->mark, &iter);
 }
 
 static gint
-iiter_compare (IAnjutaIterable *iter, IAnjutaIterable *other_iter, GError **e)
+iiter_compare (IAnjutaIterable *iiter, IAnjutaIterable *iother_iter, GError **e)
 {
-	SourceviewCell* cell = SOURCEVIEW_CELL(iter);
-	SourceviewCell* other_cell = SOURCEVIEW_CELL(other_iter);
-	
-	return gtk_text_iter_compare (cell->priv->iter, other_cell->priv->iter);
+	SourceviewCell* cell = SOURCEVIEW_CELL(iiter);
+	SourceviewCell* other_cell = SOURCEVIEW_CELL(iother_iter);
+	GtkTextIter iter;
+	GtkTextIter other_iter;
+	gtk_text_buffer_get_iter_at_mark (cell->priv->buffer, &iter, cell->priv->mark);
+	gtk_text_buffer_get_iter_at_mark (other_cell->priv->buffer, &other_iter, other_cell->priv->mark);
+
+	return gtk_text_iter_compare (&iter, &other_iter);
 }
 
 static gint
-iiter_diff (IAnjutaIterable *iter, IAnjutaIterable *other_iter, GError **e)
+iiter_diff (IAnjutaIterable *iiter, IAnjutaIterable *iother_iter, GError **e)
 {
-	SourceviewCell* cell = SOURCEVIEW_CELL(iter);
-	SourceviewCell* other_cell = SOURCEVIEW_CELL(other_iter);
+	SourceviewCell* cell = SOURCEVIEW_CELL(iiter);
+	SourceviewCell* other_cell = SOURCEVIEW_CELL(iother_iter);
+
+	GtkTextIter iter;
+	GtkTextIter other_iter;
+	gtk_text_buffer_get_iter_at_mark (cell->priv->buffer, &iter, cell->priv->mark);
+	gtk_text_buffer_get_iter_at_mark (other_cell->priv->buffer, &other_iter, other_cell->priv->mark);
 	
-	return (gtk_text_iter_get_offset (other_cell->priv->iter) 
-					- gtk_text_iter_get_offset (cell->priv->iter));
+	return (gtk_text_iter_get_offset (&other_iter) 
+					- gtk_text_iter_get_offset (&iter));
 }
 
 static void
@@ -333,6 +316,5 @@ iiter_iface_init(IAnjutaIterableIface* iface)
 
 ANJUTA_TYPE_BEGIN(SourceviewCell, sourceview_cell, G_TYPE_OBJECT);
 ANJUTA_TYPE_ADD_INTERFACE(icell, IANJUTA_TYPE_EDITOR_CELL);
-ANJUTA_TYPE_ADD_INTERFACE(icell_style, IANJUTA_TYPE_EDITOR_CELL_STYLE);
 ANJUTA_TYPE_ADD_INTERFACE(iiter, IANJUTA_TYPE_ITERABLE);
 ANJUTA_TYPE_END;
